#include <Arduino.h>
#include <util/myDelay.h>
#include <math.h> 
#include "ASM5915.h"



ASM5915::ASM5915(TwoWire &wire, int address) : 
lastInitTime_(0), wire_(wire), address_(address)
{
}


void ASM5915::begin()
{
  wire_.begin();
}


void ASM5915::initSensorValues()
{
  if(millis() - lastInitTime_  < 50)
    return;
  lastInitTime_ =   millis();

  uint32_t ret = 0;
  wire_.beginTransmission(address_);
  wire_.requestFrom(address_, (int)4);
  for(int i=0; i < 1000 && wire_.available() < 4; i++)
    delayMili(1);
  for(int i = 0; i < 4; i++){
    ret *= 256;  
    ret += wire_.read();
  }
  wire_.endTransmission();

  uint16_t rawP=0;
  uint16_t rawT=0;
  uint32_t data = ret;    
  rawT = ((uint16_t) data) >>5;
  rawP = (uint16_t) (data >> 18);

  pressure_ = -(1638.-rawP)/13107*5 + 0.46;
  temperature_ = 200.*rawT/2048-50;
}

float ASM5915::getTemperature()
{
  initSensorValues();
  return temperature_;
}



long  ASM5915::getPressure()
{
  initSensorValues();
  return (long)(pressure_*100000);
}




