// KalmanFilter
#include "KalmanFilter.h"
#include <limits> 

KalmanFilter::KalmanFilter(float q, float r) : q_(q), r_(r), p_(1.f), x_(-1.f), isFirstCall_(true)
{
}

KalmanFilter::KalmanFilter()  : q_(0.0001f), r_(0.01f), p_(1.f), x_(-1.f), isFirstCall_(true)
{
}

void KalmanFilter::update(float measurement){
  if(isFirstCall_){
    isFirstCall_ = false;
    x_ = measurement;
  }
  else{
    measurementUpdate();
    x_ = x_ + (measurement - x_) * k_;
  }
}

float KalmanFilter::get()
{
  return x_;
}

void KalmanFilter::measurementUpdate(){
  k_ = 1- r_/(p_ + q_ + r_);
  p_ = r_ * k_;
}


DfFilter::DfFilter()  : dataCnt_(0), dataSum_(0)
{
}

void DfFilter::update(float measurement)
{
  dataCnt_++;
  dataSum_ +=measurement;
}

float DfFilter::get()
{
  if(dataCnt_ > 0) {
    dataSum_ = (float)(dataSum_/dataCnt_);
    dataCnt_ = 1;
  }
  return dataSum_;
}

