#include <Arduino.h>
#include <math.h> 
#include "MS5611.h"
#include "EEProm.h"



MS5611::MS5611(TwoWire &wire, int address) : 
  lastInitTimeTemp_(0), wire_(wire), address_(address) 
{
  for (size_t i=0; i <6;i++)
    C[i] = 0;
}


void MS5611::begin()
{

  wire_.begin();
  wire_.beginTransmission(address_);
  wire_.write(0x1E); // reset
  wire_.endTransmission();
  delayMili(10);


  for (int i=0; i<6  ; i++) {
    wire_.beginTransmission(address_);
    wire_.write(0xA2 + (i * 2));
    wire_.endTransmission();

    wire_.beginTransmission(address_);
    wire_.requestFrom(address_,  6);
    delayMili(1);
    if(wire_.available()) {
      C[i] = wire_.read() << 8 | wire_.read();
    }
  }

  for(int i=0; i < 6; i++)
  {
    unsigned int value = EEPROM.read(i);
    value = (value << 8) + EEPROM.read(i+1);
  }
}


void MS5611::initSensorValues()
{
  double D1 = getVal(0x48);   // Pressure raw
  if(lastInitTimeTemp_ == 0 || millis() - lastInitTimeTemp_  > 1000){
    lastInitTimeTemp_ = millis();
    rawT_ = getVal(0x58);   // Temperature raw
  }
  
  double dT = rawT_ - ((double)C[4] * 256);
  double TEMP = ((double)dT * C[5]) / 8388608 +2000;
  
  double OFF  = (double )C[1] * 65536 + (dT * C[3]) / 128;
  double SENS = (double )C[0] * 32768 + (dT * C[2]) / 256;
  
  
  double T2 = 0;
  double OFF2 = 0;
  double SENS2 = 0;
  if (TEMP < 2000) {
    T2 = ((dT / 32768) * (dT / 32768)) / 2;
    OFF2 = 5 * (TEMP - 2000) * (TEMP - 2000) / 2;
    SENS2 = 5 * (TEMP - 2000) * (TEMP - 2000) / 4;
    if (TEMP < -1500){
        OFF2 = OFF2 + 7 * (TEMP + 1500) * (TEMP + 1500);
        SENS2 = SENS2 + 11 * (TEMP + 1500) * (TEMP + 1500) / 2;
    }
  }
  TEMP = TEMP - T2;
  OFF = OFF - OFF2;
  SENS = SENS - SENS2;
  temperature_ = (float) TEMP/ 100;
  pressure_  = (float) (( D1 * SENS / 2097152 - OFF) / 32768) / 100;
}

float MS5611::getTemperature()
{
  return temperature_;
}


long  MS5611::getPressure()
{
  initSensorValues();
  return (long)(pressure_*100);
}


uint32_t MS5611::getVal(char code)
{
  uint32_t ret = 0;
  Wire.beginTransmission(address_);
  Wire.write(code);
  Wire.endTransmission();
  delayMili(10);
  // start read sequence
  Wire.beginTransmission(address_);
  Wire.write((char) 0x00);
  Wire.endTransmission();
  Wire.beginTransmission(address_);
  Wire.requestFrom(address_, (int)3);
  if (Wire.available() >= 3)
  {
    ret = Wire.read() * (unsigned long)65536 + Wire.read() * (unsigned long)256 + Wire.read();
  }
  else {
    ret = -1;
  }
  Wire.endTransmission();
  return ret;
}


