#include <Arduino.h>
#include "Util.h"



SyncSerial::SyncSerial(unit8_t pinTx, uint8_t pinListn) 
: 
rate_(9600), pinTx_(pinTx), pinListn_(pinListn)
{
}

void SyncSerial::begin(uint16_t rate)
{
  pinMode(pinListn_, INPUT);
  txDelay_ = (uint16_t)(1000000. / rate);
  txMiniDelay_ = max(txDelay_/25,1);
  rate_ = rate;
}


void SyncSerial::waitForFreePipe()
{
  for(uint16_t i=0; i < txDelay_*8; i+=txMiniDelay_)
  {
    delayMcro(txMiniDelay_);
    if(digitalRead(pinListn_) != HIGH) 
      i=0;  
  }
}

void SyncSerial::send(unsigned char value, unit8_t pin)
{
  byte mask;
  //startbit
  digitalWrite(pin, LOW);
  delayMcro(txDelay_);
  for (mask = 0x01; mask>0; mask <<= 1) {
    digitalWrite(pin, (value & mask) ? HIGH : LOW);
    delayMcro(txDelay_);
  }
  //stop bit
  digitalWrite(pin, HIGH);
  delayMcro(txDelay_);
}

void SyncSerial::send(const String &data, unit8_t pin, boolean sync)
{
  if(pin == 0xFF)
    pin = pinTx_;
  if(sync) {
    waitForFreePipe();
  }
  pinMode(pin, OUTPUT);

  for(int i=0; i < data.length(); i++){
    send((unsigned char )data[i], pin);
  }
  if(sync) {
    waitForFreePipe();
  }
  pinMode(pin, INPUT);  

}

BeepModul::BeepModul(const Vector<int> &pins) : 
pins_(pins), level_(-1), frequence_(1000), duration_(500)
{

}

void BeepModul::setNextLevel(){
  for(int i=0; i < pins_.size(); i++)
    pinMode(pins_[i],INPUT);
  level_++;
  if (level_ == pins_.size()) {
    pinMode(pins_[level_-1],OUTPUT);
    tone(pins_[level_-1], 1000, 250);
    delayMili(500);
    tone(pins_[level_-1], 1000, 250);
    delayMili(500);
    pinMode(pins_[level_-1],INPUT);
    level_ = -1;
  }else{
    pinMode(pins_[level_],OUTPUT);
    tone(pins_[level_], 1000, 500);
  }
}

void BeepModul::beep(){
  if(duration_ != 0 && frequence_ != 0 && level_!= -1)
    tone(pins_[level_], frequence_, duration_);
}

void BeepModul::setSpeed(float speedM_S){
  if(speedM_S < -2){
    frequence_ = 400;
    duration_ = (int)((-2 - speedM_S))*500+90;
  }
  else if(speedM_S > 0.5){
    frequence_ = 2000;
    duration_ = (int)((speedM_S - 0.5))*500+90;
  }
  else{
    frequence_=0;
  }
  if(duration_ > 500)
    duration_ = 500;
}





