#ifndef __UTIL_H__
#define __UTIL_H__
#include <Arduino.h>
#include <math.h> 
#include <string>

typedef unsigned char  unit8_t;

// Minimal class to replace std::vector
template<typename Data>
class Vector {
   size_t d_size; // Stores no. of actually stored objects
   size_t d_capacity; // Stores allocated capacity
   Data *d_data; // Stores data
   public:
     Vector() : d_size(0), d_capacity(0), d_data(0) {}; // Default constructor
     Vector(Vector const &other) : d_size(other.d_size), d_capacity(other.d_capacity), d_data(0) { d_data = (Data *)malloc(d_capacity*sizeof(Data)); memcpy(d_data, other.d_data, d_size*sizeof(Data)); }; // Copy constuctor
     Vector(size_t pSize, const Data  other[]) : d_size(pSize), d_capacity(pSize), d_data(0) { d_data = (Data *)malloc(d_capacity*sizeof(Data)); memcpy(d_data, other, d_size*sizeof(Data)); }; // Copy constuctor
     ~Vector() { free(d_data); }; // Destructor
     Vector &operator=(Vector const &other) { free(d_data); d_size = other.d_size; d_capacity = other.d_capacity; d_data = (Data *)malloc(d_capacity*sizeof(Data)); memcpy(d_data, other.d_data, d_size*sizeof(Data)); return *this; }; // Needed for memory management
     void push_back(Data const &x) { if (d_capacity == d_size) resize(); d_data[d_size++] = x; }; // Adds new value. If needed, allocates more space
     size_t size() const { return d_size; }; // Size getter
     Data const &operator[](size_t idx) const { return d_data[idx]; }; // Const getter
     Data &operator[](size_t idx) { return d_data[idx]; }; // Changeable getter
   private:
     void resize() { d_capacity = d_capacity ? d_capacity*2 : 1; Data *newdata = (Data *)malloc(d_capacity*sizeof(Data)); memcpy(newdata, d_data, d_size * sizeof(Data)); free(d_data); d_data = newdata; };// Allocates double the old space
};

inline static void delayMcro(uint16_t delay){
  uint8_t tmp=0;
  asm volatile( "sbiw    %0, 0x01 \n\t"
                "ldi %1, 0xFF \n\t"
                "cpi %A0, 0xFF \n\t"
                "cpc %B0, %1 \n\t"
                "brne .-10 \n\t"
                : "+r" (delay), "+a" (tmp)
                : "0" (delay)
              );
}

inline static void delayMili(uint16_t ms){
  if(ms < 65)
    delayMcro(ms*1000);
  else
    delay(ms);
}


class BarometerIcf{
public:
  virtual float getTemperature() = 0;
  virtual long  getPressure() = 0;
  float calcAltitude(long pressure, long QNH)
  {
    float A = ((float)pressure)/QNH;
    float B = 1/5.25588;
    float C = pow(A,B);
    C = 1 - C;
    C = C /0.0000225577;
    return C;
  }

};


class Akku {
public:
  inline static long readVcc(){
    long result = -1;
    // Read 1.1V reference against AVcc
    ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
    delayMili(2); // Wait for Vref to settle
    ADCSRA |= _BV(ADSC); // Convert
    while (bit_is_set(ADCSRA,ADSC));
    result = ADCL;
    result |= ADCH<<8;
    result = 1126400L / result; // Back-calculate AVcc in mV
    return result;
  }  
};


class SyncSerial{
  uint16_t rate_; 
  uint16_t txDelay_; 
  uint16_t txMiniDelay_; 
  unit8_t  pinTx_; 
  uint8_t  pinListn_;
  void send(unsigned char value, unit8_t pin);
  void waitForFreePipe();
public:
  SyncSerial(unit8_t pinTx, uint8_t pinListn);
  void send(const String &data, unit8_t pin = 0xFF, boolean sync = true);
  void begin(uint16_t rate);
};

class BeepModul{
  Vector<int> pins_;
  int   level_;
  
  int   frequence_;
  int   duration_;
public:
  BeepModul(const Vector<int> &pins);
  void setNextLevel();
  void beep();
  void setSpeed(float speedM_S);
};



#endif //__UTIL_H__
